/*	Program to format image header parameters for telescope data.
 *	This routine extracts parameters from image headers and makes a
 *	list of strings that can be picked up for the master log program

 *	Author:	Tod R. Lauer	4/9/94	
 *	Modified to read from image file by Doug Mink, 2001-05-11
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

extern int add_fast(), add_ech(), add_ccd();

#define MAXC 640		/* Length of input line.		*/
#define MAXW 640		/* Maximum word length.			*/

int
main (ac,av)

int ac;
char *av[];
{
	int	verbose, err;
	char	*imagefile, *instrument, *avi;

	if (ac < 2) {
            fprintf (stderr, "add_log: Write header info to stdout for log\n");
            fprintf (stderr, "Usage: add_log [-v] instrument image_name\n");
            fprintf (stderr, "       -v prints what is being done\n");
            fprintf (stderr, "       instrument is fast echelle 1sh 4sh\n");
            fprintf (stderr, "       image_name is FITS or .imh image\n");
	    exit (-1);
	    }

	verbose = 0;
	instrument = NULL;
	imagefile = NULL;

	for (av++; --ac > 0; av++) {
        /* Instrument is first command line argument */
            avi = *av;
	    if (avi[0] == '-' && avi[1] == 'v')
		verbose = 1;
	    else if (instrument == NULL)
		instrument = avi;
	    else if (imagefile == NULL)
		imagefile = avi;
	    }
        if (instrument == NULL && imagefile == NULL) {
            fprintf (stderr, "ADD_LOG: No instrument or image file\n");
            exit (-1);
            }
        else if (instrument == NULL) {
            fprintf (stderr, "ADD_LOG: No instrument\n");
            exit (-1);
            }
        else if (imagefile == NULL) {
            fprintf (stderr, "ADD_LOG: No image file\n");
            exit (-1);
            }

	/* Call appropriate log addition subroutine for instrument */
	if (strcmp(instrument,"direct") == 0)
	    err = add_ccd (imagefile);
	else if (strcmp(instrument,"1sh") == 0)
	    err = add_ccd (imagefile);
	else if (strcmp(instrument,"4sh") == 0)
	    err = add_ccd (imagefile);
	else if (strcmp(instrument,"mosaic") == 0)
	    err = add_ccd (imagefile);
	else if (strcmp(instrument,"ech") == 0)
	    err = add_ech (imagefile);
	else if (strcmp(instrument,"fast") == 0)
	    err = add_fast (imagefile, verbose);
	else {
	    fprintf (stderr, "Unknown instrument: %s\n", instrument);
	    exit (-1);
	}
	exit (err);
}

/* Jan 16 2007	(Doug Mink) Find verbose flag in any position
 */
