/* File obslog$Src/init_log.c
 * January 12, 2007
 *
 * Setup formatted TeX logfile headers */

/* FAST, Echelle, and CCD version */

/* Author: Tod R. Lauer 9/23/96 */
/* Rewritten by Doug Mink, May 10, 2001 */

#define MAXC 5000                /* Length of input line */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include "fitsfile.h"
#include "wcstab.h"

int
main (argc,argv)
int argc;
char *argv[];
{
	FILE    *hp, *pp;

	char s[MAXC], file[MAXC];
	char *line;
        struct TabTable *logtab;
        struct Tokens tokens;   /* Token structure returned */
	char *imagefile;
	char *instrument;
	char *header;
	char focus[64];
	char *GetFITShead();

	int page;
	/*
	fprintf (stderr, "INIT_LOG: arg 0: %s\n", argv[0]);
	fprintf (stderr, "INIT_LOG: arg 1: %s\n", argv[1]);
	fprintf (stderr, "INIT_LOG: arg 2: %s\n", argv[2]);
	*/

        /* Page number is first command line argument */
        if (argc > 1)
            page = atoi (argv[1]);
        else {
            fprintf (stderr, "init_log: Write TeX log header files\n");
            fprintf (stderr, "Usage: init_log first_page instrument image_file\n");
            fprintf (stderr, "       instrument is fast ech 1sh 4sh\n");
            exit (-1);
	    }
	/* fprintf (stderr, "INIT_LOG: page %d\n", page); */

        /* Instrument is second command line argument */
        if (--argc > 0)
            instrument = argv[2];
        else {
	    fprintf (stderr, "INIT_LOG: No instrument %s\n", argv[2]);
            exit (-1);
	    }

        /* Image file is third command line argument */
        if (--argc > 0)
            imagefile = argv[3];
        else {
	    fprintf (stderr, "INIT_LOG: No data file %s\n", argv[3]);
            exit (-1);
	    }

	/* Read first line of tab table log to get date and constant parameters */
	if (!strncmp (instrument, "h", 1)) {
	    if ((logtab = tabopen (imagefile, 0)) == NULL) {
		fprintf (stderr, "INIT_LOG: Cannot read Starbase file %s\n", imagefile);
        	exit (-1);
		}
	    }

	/* Read FITS header of sample image to get date and constant parameters */
	else {
	    if ((header = GetFITShead (imagefile)) == NULL) {
		fprintf (stderr, "INIT_LOG: Cannot read image header %s\n", imagefile);
        	exit (-1);
		}
	    }

	/* Save the first page number in "logpage.tex if the file does not exist */
	strcpy (file, "logpage.tex");
	if (access (file, F_OK)) {
	    if (open (file, O_WRONLY | O_CREAT | O_TRUNC, 0644) == -1) {
		printf ("Error creating page file %s\n", file);
		exit (1);
      		}
	    if ((pp = fopen (file,"w")) == NULL)
		printf ("Error opening page file %s\n", file);
	    sprintf (s, "%d", page);
	    fprintf(pp,"\\pageno=%s\n", s);
	    (void) fclose (pp);
	    }

	/* The header is in TeX and is saved in "loghead.tex */
	strcpy (file, "loghead.tex");
	if (creat (file, 0644) == -1) {
	    printf ("Error creating header file %s\n", file);
	    exit (1);
	    }
	if ((hp = fopen (file,"w")) == NULL)
	    printf ("Error opening loghead.tex\n");

	fprintf(hp,"\\rm\\null\\medskip\n");
	fprintf(hp,"\\noindent\n");

	/* UT Date */
	if (!strncmp (instrument, "h", 1)) {
	    line = gettabline (logtab, 1);
	    setoken (&tokens, line, "tab");
	    if (tabgetk (logtab, &tokens, "DATE-OBS", s, MAXC))
		s[0] = (char) 0;
            fprintf (stderr, "init_log: Start date is %s\n", s);
	    }
	else
	    hgets (header, "DATE-OBS", MAXC, s);
	texcheck (s);
	fprintf(hp,"\\hbox to\\hsize{\n");
	fprintf(hp,"{\\kbigit UT Date\\ }\\underbar{\\hbox to 0.85in{\\kxztf\\thinspace %s\\hfil}}\n", s);

	/* Observers */
	strcpy (s, "none");
	if (!strncmp (instrument, "h", 1)) {
	    if (tabgetk (logtab, &tokens, "OBSERVERS", s, MAXC))
		s[0] = (char) 0;
            fprintf (stderr, "init_log: Observers are %s\n", s);
	    }
	else
            hgets (header, "OBSERVER", MAXC, s);
	texcheck (s);
	texlong (s, 30);
	fprintf(hp,"\\quad {\\kbigit Observers\\ }\\underbar{\\hbox to 2.6in{\\kxztf\\thinspace %s\\hfil}}\\hfill\n", s);

	/* For CCD, write binning and instrument ID in header */
	if (!strcmp (instrument, "1sh") || !strcmp (instrument, "4sh")) {
	    strcpy (s, "1 1");
	    hgets (header, "CCDSUM", MAXC, s);
	    texcheck(s);
	    fprintf(hp,"\\quad {\\kbigit Binning\\ }\\underbar{\\hbox to 0.5in{\\kxztf\\thinspace %s\\hfil}}\n", s);

	    hgets (header, "DETECTOR", MAXC, s);
	    texcheck (s);
	    fprintf(hp,"{\\kbigit CCD\\ }\\underbar{\\hbox to 0.70in{\\kxztf\\thinspace %s\\hfil}}\n", s);

	    strcpy (s, "none");
	    hgets (header, "PROGRAM", MAXC, s);
	    texcheck(s);
	    texlong(s, 40);
	    fprintf(hp,"{\\kbigit Program\\ }\\underbar{\\hbox to 3.6in {\\kxztf\\thinspace %s\\hfil}}}\n", s);
	    }

	/* For FAST, write focus and grating in header */
	else if (!strcmp (instrument, "fast")) {
	    strcpy (focus, "1000");
	    if (!hgets (header, "FASTFOC", 64, focus))
		hgets (header, "INSFOC", 64, focus);
	    texcheck (focus);
	    texlong (focus, 4);
	    fprintf(hp,"{\\kbigit Coll.~Focus\\ }\\underbar{\\hbox to 0.40in {\\kxztf\\hfil {\\kpnosmr\\ksmtt %s}\\hfil}}\n", focus);

	    strcpy (s, "300");
	    hgets (header, "DISPERSE", 64, s);
	    texcheck (s);
	    texlong(s, 6);
	    fprintf(hp,"\\quad{\\kbigit Grating/Grism\\ }\\underbar{\\hbox to 0.6in {\\kxztf\\hfil{\\kpnosmr\\ksmttt %s}\\hfil}}\\hfill}\n", s);
	    }

	/* For Hectospec write instrument in header */
	else if (!strncmp (instrument, "h", 1)) {
	    strcpy (s, "300");
	    if (tabgetk (logtab, &tokens, "DISPERSER", s, MAXC) < 1)
		s[0] = (char) 0;
	    texcheck (s);
	    texlong(s, 6);
	    fprintf(hp,"\\quad{\\kbigit Grating/Grism\\ }\\underbar{\\hbox to 0.6in {\\kxztf\\hfil{\\kpnosmr\\ksmttt %s}\\hfil}}\\hfill}\n", s);
	    }

	(void) fclose (hp);
	exit (0);
}
/* Apr 18 2001	Write to initialize TeX log files for FAST
 * Apr 20 2001	Update to deal with any instrument
 * May 10 2001	Change name to init_log.c; add CCD information
 * May 11 2001	Initialize strings read from header parameters
 * May 30 2001	Do not write logpage.tex if it already exists
 *
 * Nov 13 2002	Add FASTFOC keyword to replace INSFOC
 *
 * Mar  3 2004	Add Hectospec and Hectochelle
 *
 * Jan 12 2007	Include unistd.h
 */
