/* tex_hspec.c
 * Write TeX file from Starbase Hectospec log, returning number of pages written
 * By Doug Mink, January 12, 2007
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include "fitsfile.h"
#include "wcstab.h"

#define MAXC 640		/* Length of input line */
#define MAXB 5000		/* Length of output buffer */
#define MAXW 64			/* Maximum word length	*/


int
tex_hspec (logroot, libdir, debug)

char	*logroot;	/* Rootname of log file */
char	*libdir;	/* Pathname of obslog lib directory */
int	debug;		/* If != 0, print debugging information */

{
	int	i, nline, npage;
	int	ntok, nlog, ntok0, nsame;
	FILE	*log, *tex;
	struct TabTable *logtab;
	struct Tokens tokens;	/* Token structure returned */
	int	iexp, iuts, ira, idec, igrate, ifn, iprog, icom, ipa;
	double dec, dec0;
	int	last = 0;

	/* individual parameters from log file */
	char	sname[MAXW],	/* Spectrum file name root */
		snum[MAXW],	/* Sequence number for night */
		sobj[MAXW],	/* Object name */
		sexp[MAXW],	/* Exposure time */
		sst[MAXW],	/* Start time */
		sra[MAXW],	/* Object right ascension */
		sdec[MAXW],	/* Object declination */
		sgrate[MAXW],	/* Spectrograph grating */
		sprog[MAXW],	/* Program number */
		spa[MAXW],	/* Spectrograph position angle */
		sbin[MAXW],	/* CCD binning */
		scoma[MAXW];	/* First line of comment */
	char	sname0[MAXW],	/* Spectrun file name root */
		snum0[MAXW],	/* Sequence number for night */
		snum1[MAXW],	/* Sequence number for night (1st of dupes) */
		sobj0[MAXW],	/* Object name */
		sexp0[MAXW],	/* Exposure time */
		sst0[MAXW],	/* Start time */
		sra0[MAXW],	/* Object right ascension */
		sdec0[MAXW],	/* Object declination */
		sgrate0[MAXW],	/* Spectrograph grating */
		sprog0[MAXW],	/* Program number */
		spa0[MAXW],	/* Spectrograph position angle */
		sbin0[MAXW],	/* CCD binning */
		scoma0[MAXW];	/* First line of comment */
	char	inline1[MAXC],
		word[MAXW],
		texw[MAXW],
		file[MAXC],
	        file2[MAXC];
	char	outline[MAXB];
	char	*oline, *olast;
	char	czero = (char) 0;
	char	*nextline;

	/* fprintf (stderr, "TEX_HSPEC: creating %s.tex\n", logroot); */

	/* Create TeX output file */
	strcpy (file, logroot);
	strcat (file, ".tex");
	if (open (file, O_WRONLY | O_CREAT | O_TRUNC, 0644) == -1) {
	    fprintf (stderr, "Error creating TeX file %s\n", file);
	    return (0);
	    }
 
	if ((tex = fopen (file,"w")) == NULL) { 
	    fprintf (stderr, "Error opening  TeX file %s\n", file); 
	    return (0);
	    } 
 
	/* Put in standard header stuff and first page number */
	strcpy (file, libdir);
	strcat (file, "/");
	strcat (file, "kptab1hspec");
	fprintf (tex, "\\input %s\n", file);
	fprintf (tex, "\\input logpage\n");
	fprintf (tex, "\\offinterlineskip\n");

	/* Now open the log file */
	strcpy (file2, logroot);
	strcat (file2, ".tab");
	if ((logtab = tabopen (file2, 0)) == NULL) {
	    fprintf (tex, "\\noindent Error opening main logfile %s\n", file2);
	    fprintf (tex, " --- No observations recorded yet?\\par\n");
	    fprintf (tex, "\\vfill}\\eject\n\\end\n");
	    return (0);
	    }

	/* Get column numbers of desired parameters for quick parsing */
	iexp = tabcol (logtab, "EXPTIME");
	iuts = tabcol (logtab, "UT");
	ira = tabcol (logtab, "RA");
	idec = tabcol (logtab, "DEC");
	igrate = tabcol (logtab, " DISPERSER");
	ifn = tabcol (logtab, "FILENAME");
	iprog = tabcol (logtab, "PROPID");
	icom = tabcol (logtab, "COMMENTS");
	ipa = tabcol (logtab, "TILTPOS");
	if (debug) {
	    fprintf (stderr,"iexp=%d, iuts=%d, ira=%d, idec=%d\n",iexp,iuts,ira,idec);
	    fprintf (stderr,"igrate=%d, ifn=%d, iprog=%d, icom=%d\n",igrate,ifn,iprog,icom);
	    }

	/* Get lines until end of log file is reached */
	nline = 0;
	nlog = 0;
	nsame = 0;
	last = 0;
	while ((nextline = gettabline (logtab, nlog)) != NULL || !last) {

	    if (debug)
	        fprintf (stderr,"line %d %d: %s\n", nlog+1, last, nextline);
	    if (nextline == NULL)
		last = 1;

	    /* Get info for this line */
	    if (!last) {
		nlog++;

		/* Parse next line from log file */
		setoken (&tokens, nextline, "tab");
		ntok = tokens.ntok;

		/* File name */
		(void) getoken (&tokens, ifn, sname, MAXW);
		strcpy (sobj, sname+5);
		strncpy (snum, sname, 4);
		snum[4] = (char) 0;

		(void) getoken (&tokens, iexp, sexp, MAXW); /* Exposure time */
		(void) getoken (&tokens, iuts, sst, MAXW); /* UT Start time */
		(void) getoken (&tokens, ira, sra, MAXW); /* Right Ascension */
		(void) getoken (&tokens, idec, sdec, MAXW); /* Declination */
		dec = str2dec (sdec);
		(void) getoken (&tokens, igrate, sgrate, MAXW);	/* Grating */
		(void) getoken (&tokens, ipa, spa, MAXW); /* Position Angle */
		(void) getoken (&tokens, iprog, sprog, MAXW); /* Program Name */
		(void) getoken (&tokens, icom, scoma, MAXW); /* Comments */
		}

	    /* First time through, just save parameters */
	    if (nlog == 1) {
		strcpy (sname0, sname);
		strcpy (snum0, snum);
		strcpy (sobj0, sobj);
		strcpy (sexp0, sexp);
		strcpy (sst0, sst);
		strcpy (sgrate0, sgrate);
		strcpy (sra0, sra);
		strcpy (sdec0, sdec);
		strcpy (spa0, spa);
		strcpy (sprog0, sprog);
		strcpy (scoma0, scoma);
		dec0 = dec;
		ntok0 = ntok;
		if (!last)
		    continue;
		}

	    /* Compare to previous line */
	    else if ((!strcasecmp (sobj, "DARK") ||
		      !strcasecmp (sobj, "BIAS") ||
		      !strcasecmp (sobj, "FLAT") ||
		      !strcasecmp (sobj, "SKY")) &&
		      !last && dec == dec0 &&
		     (!strcmp (sobj, sobj0) && !strcmp (sexp, sexp0) &&
		      !strcmp (sgrate, sgrate0) && !strcmp (sbin, sbin0) &&
		      !strcmp (scoma, scoma0) )) {
		nsame++;
		if (nsame == 1)
		    strcpy (snum1,snum0);
		strcpy (sname0, sname);
		strcpy (snum0, snum);
		strcpy (sobj0, sobj);
		strcpy (sexp0, sexp);
		strcpy (sst0, sst);
		strcpy (sgrate0, sgrate);
		strcpy (sra0, sra);
		strcpy (sdec0, sdec);
		strcpy (spa0, spa);
		strcpy (sprog0, sprog);
		strcpy (scoma0, scoma);
		dec0 = dec;
		ntok0 = ntok;
		if (!last)
		    continue;
		}

	    /* If not same as previous spectrum, print buffered spectrum */

	    /* Print ranges of sequences numbers and start times */
	    if (nsame > 0)
		sprintf (sname0, "%4s-%4s.%s",snum1, snum0, sobj0);

	    nsame = 0;

	    /* Finish page after 20th line */
	    if (nline == 20) {
		fprintf (tex, "}\\vfill}\\eject\n");
		nline	= 0;
		}

	    /* Print page header on first line */
	    if (nline == 0) {
	        fprintf (tex, "\\vbox{\\null\\vskip 3.7pt\n");
		fprintf (tex, "\\input loghead\n");
	        fprintf (tex, "\\vskip 7pt\n");
		strcpy (file, libdir);
		strcat (file, "/");
		strcat (file, "sao.logo.ps");
		fprintf (tex,
		    "\\special{psfile=%s hoffset=-253 voffset=-340}\n",file);
		strcpy (file, libdir);
		strcat (file, "/");
		strcat (file, "spec.eps");
		fprintf (tex, "\\special{psfile=%s hoffset=453 voffset=-60}\n",file);
		strcpy (file, libdir);
		strcat (file, "/");
		strcat (file, "kptab2hspec");
		fprintf (tex, "\\input %s\n", file);
		}

	    /* Write output TeX line */
	    nline++;

	    olast = outline + MAXB;
	    for (oline = outline; oline < olast; oline++)
		*oline = czero;

	    /* File name */
	    texlong (sname0, 19); 
	    sprintf (texw,"\\ %s&", sname0);
	    strcpy (outline, texw);

	    /* Exposure time */
	    sprintf (texw,"%s\\thinspace&", sexp0);
	    strcat (outline, texw);

	    /* UT Start time */
	    sprintf (texw,"%s\\thinspace&", sst0);
	    strcat (outline, texw);

	    /* Right Ascension */
	    sprintf (texw,"%s&", sra0);
	    strcat (outline, texw);

	    /* Declination */
	    sprintf (texw,"%s&", sdec0);
	    strcat (outline, texw);

	    /* Grating */
	    sprintf (texw,"%s&", sgrate0);
	    strcat (outline, texw);

	    /* Program Name */
	    texlong (sprog0, 18);
 	    if (strncmp (sprog0, "none", 4))
		sprintf (texw,"\\thinspace %s&", sprog0);
	    else
		sprintf (texw,"%s&", sprog0);
	    strcat (outline, texw);

	    /* Remarks */
	    strcpy (word, scoma0);
	    texlong (word, 26);
	    sprintf (texw,"\\thinspace %s&\\cr", word);
	    strcat (outline, texw);

/*      Check to make sure that we got all the words.  If we didn't,
	and error on input has occured, and the line must be completed  */
 
	    for (i = ntok; i < 8; i++)
		strcat (outline," &");
	    strcat (outline, "\\cr");

	    /* Print this line of the log */
	    fprintf (tex, "%s\n", outline);
	    fprintf (tex, "\\noalign{\\khrule}\n");

	    if (last)
		break;

	    /* Save latest values */
	    strcpy (sname0, sname);
	    strcpy (snum0, snum);
	    strcpy (sobj0, sobj);
	    strcpy (sst0, sst);
	    strcpy (sexp0, sexp);
	    strcpy (sgrate0, sgrate);
	    strcpy (sra0, sra);
	    strcpy (sdec0, sdec);
	    strcpy (sprog0, sprog);
	    strcpy (scoma0, scoma);
	    dec0 = dec;
	    inline1[0] = (char) 0;
	    }

	fprintf (tex, "}\\vfill}\\end\n");
	(void) fclose (tex);
	(void) fclose (log);
	return (npage);
}
/* Mar  4 2004	New subroutine, based on tex_fast.c
 *
 * Nov 15 2005	Change inline to inline1 for Linux
 *
 * Jan 11 2006	Changed debug argument from char* to int
 * Jan 11 2006	Add more include files; change creat() to open()
 * Jan 12 2007	Pass pointer to token string structure to getoken()
 */
